﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Sailboat.DrawingObjects;
using Sailboat.MovementStrategy;

namespace Sailboat.Generics
{
    internal class BoatsGenericStorage
    {
        /// <summary>
        /// Словарь (хранилище)
        /// </summary>
        readonly Dictionary<string, BoatsGenericCollection<DrawingBoat, DrawingObjectBoat>> _boatStorages;
        /// <summary>
        /// Возвращение списка названий наборов
        /// </summary>
        public List<string> Keys => _boatStorages.Keys.ToList();
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;
        /// <summary>
        /// Разделитель для записи ключа и значения элемента словаря
        /// </summary>
        private static readonly char _separatorForKeyValue = '|';
        /// <summary>
        /// Разделитель для записей коллекции данных в файл
        /// </summary>
        private readonly char _separatorRecords = ';';
        /// <summary>
        /// Разделитель для записи информации по объекту в файл
        /// </summary>
        private static readonly char _separatorForObject = ':';
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="pictureWidth"></param>
        /// <param name="pictureHeight"></param>
        public BoatsGenericStorage(int pictureWidth, int pictureHeight)
        {
            _boatStorages = new Dictionary<string,
            BoatsGenericCollection<DrawingBoat, DrawingObjectBoat>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        /// <summary>
        /// Добавление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void AddSet(string name)
        {
            if (_boatStorages.ContainsKey(name))
            {
                return;
            }
            _boatStorages[name] = new BoatsGenericCollection<DrawingBoat, DrawingObjectBoat>(_pictureWidth, _pictureHeight);
        }
        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void DelSet(string name)
        {
            if (!_boatStorages.ContainsKey(name))
            {
                return;
            }
            _boatStorages.Remove(name);
        }
        /// <summary>
        /// Доступ к набору
        /// </summary>
        /// <param name="ind"></param>
        /// <returns></returns>
        public BoatsGenericCollection<DrawingBoat, DrawingObjectBoat>? this[string ind]
        {
            get
            {
                if (_boatStorages.ContainsKey(ind))
                {
                    return _boatStorages[ind];
                }
                return null;
            }
        }

        /// <summary>
        /// Сохранение информации по автомобилям в хранилище в файл
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - сохранение прошло успешно, false - ошибка при сохранении данных</returns>
        public bool SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            StringBuilder data = new();
            foreach (KeyValuePair<string, BoatsGenericCollection<DrawingBoat, DrawingObjectBoat>> record in _boatStorages)
            {
                StringBuilder records = new();
                foreach (DrawingBoat? elem in record.Value.GetBoats)
                {
                    records.Append($"{elem?.GetDataForSave(_separatorForObject)}{_separatorRecords}");
                }
                data.AppendLine($"{record.Key}{_separatorForKeyValue}{records}");
            }
            if (data.Length == 0)
            {
                return false;
            }
            using (StreamWriter writer = new StreamWriter(filename))
            {
                writer.Write($"PlaneStorage{Environment.NewLine}{data}");
            }
            return true;
        }
        /// <summary>
        /// Загрузка информации по автомобилям в хранилище из файла
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - загрузка прошла успешно, false - ошибка призагрузке данных</returns>
        public bool LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                return false;
            }

            using (StreamReader fs = File.OpenText(filename))
            {
                string str = fs.ReadLine();
                if (str == null || str.Length == 0)
                {
                    return false;
                }
                if (!str.StartsWith("BoatStorage"))
                {
                    return false;
                }

                _boatStorages.Clear();
                string strs = "";

                while ((strs = fs.ReadLine()) != null)
                {
                    if (strs == null)
                    {
                        return false;
                    }

                    string[] record = strs.Split(_separatorForKeyValue, StringSplitOptions.RemoveEmptyEntries);
                    if (record.Length != 2)
                    {
                        continue;
                    }
                    BoatsGenericCollection<DrawingBoat, DrawingObjectBoat> collection = new(_pictureWidth, _pictureHeight);
                    string[] set = record[1].Split(_separatorRecords, StringSplitOptions.RemoveEmptyEntries);
                    foreach (string elem in set)
                    {
                        DrawingBoat? boat = elem?.CreateDrawingBoat(_separatorForObject, _pictureWidth, _pictureHeight);
                        if (boat != null)
                        {
                            if (!(collection + boat))
                            {
                                return false;
                            }
                        }
                    }
                    _boatStorages.Add(record[0], collection);
                }
                return true;
            }
        }
    }
}
